/**
 * @author Vsevolod Strukchinsky @floatdrop
 */


/**
Camera class represents DisplayObjectContainer, that autotrack one of DisplayObject positions.

@example
	var stage = new PIXI.Stage();
	var layers = new LINK.Layers("game","ui");
	stage.addChild(layers);
	var camera = new LINK.Camera();
	camera.follow(player).on(layers.game);

@class Camera
@extends DisplayObjectContainer
@constructor
**/
LINK.Camera = function (width, height) {
	PIXI.DisplayObjectContainer.call(this);

	this.width = width;
	this.height = height;
	this.halfWidth = (width / 2) | 0;
	this.halfHeight = (height / 2) | 0;

	/**
	 * The bounds of that the camera can move to
	 *
	 * @property bounds
	 * @type PIXI.Rectangle
	 * @readOnly
	 * @private
	 */
	var _bounds = new PIXI.Rectangle(0, 0, 0, 0);
	Object.defineProperty(this, "bounds", {
		get: function () {
			return _bounds;
		},
		set: function (n) {
			if (!(n instanceof PIXI.Rectangle)) throw new Error(n + " bounds must be instance of PIXI.Rectangle");
			_bounds = n.clone();
			_bounds.maxX = n.x + n.width - this.width;
			_bounds.minX = n.x;
			_bounds.maxY = n.y + n.height - this.height;
			_bounds.minY = n.y;
		}
	});

	/**
	 * Freezes the camera
	 * @property freeze
	 * @type Boolean
	 */
	this.freeze = false;

	/**
	 * Freezes the camera
	 * @property freeze
	 * @type Boolean
	 * @private
	 */
	this._linkedObject = null;
};

LINK.Camera.constructor = LINK.Camera;
LINK.Camera.prototype = Object.create(PIXI.DisplayObjectContainer.prototype);

/**
 * Wraps DisplayObject to pan him and all his childrens.
 * @method on
 * @param object {DisplayObjectContainer}
 */
LINK.Camera.prototype.on = function (object) {
	if (!(object instanceof PIXI.DisplayObjectContainer)) throw new Error(object + " object should be instance of DisplayObjectContainer");

	if (object instanceof PIXI.Stage) {
		while (object.children[0]) {
			this.addChild(object.children[0]);
		}
		object.addChild(this);
		return this;
	}

	if (this.parent) {
		this.off();
	}

	var parent = object.parent;
	if (!parent) {
		throw new Error("Can't bind Camera on DisplayObjectContainer without parent");
	}
	var index = parent.children.indexOf(object);
	parent.removeChild(object);
	this.addChild(object);
	object = this;
	parent.addChildAt(object, index);
	return this;
};

/**
 * Unwraps DisplayObject that was wrapped by "on" method.
 * @method off
 */
LINK.Camera.prototype.off = function () {
	var parent = this.parent;
	if (parent) {
		var index = parent.children.indexOf(this);
		parent.removeChild(this);
		while (this.children[0]) {
			parent.addChildAt(this.children[0], index++);
		}
		return this;
	}
	throw new Error("Can't remove Camera without Camera.parent defined");
};

/**
 * Starts follow on DisplayObject
 * @method follow
 * @param object {DisplayObject}
 */
LINK.Camera.prototype.follow = function (object) {
	this._linkedObject = object;
	return this;
};

/**
@method updateTransform
@internal
*/
LINK.Camera.prototype.updateTransform = function () {
	if (this._linkedObject && !this.freeze) {
		var anchor = this._linkedObject.position.clone();
		anchor.x -= this.halfWidth;
		anchor.y -= this.halfHeight;

		anchor.x = Math.min(this.bounds.maxX, Math.max(this.bounds.minX, anchor.x));
		anchor.y = Math.min(this.bounds.maxY, Math.max(this.bounds.minY, anchor.y));

		this.position = new PIXI.Point(-anchor.x, -anchor.y);
	}
	PIXI.Sprite.prototype.updateTransform.call(this);
};