/**
 * @author Vsevolod Strukchinsky @floatdrop
 */

/**
TiledMap class loads json file of tiled map. 

@example
	var stage = new PIXI.Stage();
	var map = new LINK.TiledMap(mapUri, { layerAsCanvas: true });
	stage.addChild(map);


@class TiledMap
@constructor
**/
LINK.TiledMap = function (mapUrl, config) {

	PIXI.EventTarget.call(this);

	LINK.Layers.call(this);

	var self = this;

	this.config = config;

	this.tilecache = {};

	var loader = new LINK.JsonLoader(mapUrl, false);
	loader.addEventListener("loaded", function (jsonLoader) {
		self.onTiledMapJsonLoaded(jsonLoader);
	});
	loader.load();
};

LINK.TiledMap.constructor = LINK.TiledMap;
LINK.TiledMap.prototype = Object.create(LINK.Layers.prototype);

LINK.TiledMap.prototype.onTiledMapJsonLoaded = function (jsonLoader) {
	var map = jsonLoader.content.json;
	var self = this;

	this.scale = {
		x: parseInt(map.properties.scale, 10) || 1,
		y: parseInt(map.properties.scale, 10) || 1
	};

	this.tilesize = {
		width: map.tilewidth,
		height: map.tileheight
	};

	this.tilesets = [];

	this.tilecache = {};

	var tsLoaded = 0;
	var tilesetLoaded = function (imageLoader) {
		tsInfo.baseTexture = imageLoader.content.texture.baseTexture;
		tsLoaded += 1;
		if (tsLoaded === map.tilesets.length) {
			self.tilesetsLoaded(map);
		}
	};

	for (var j = 0, jl = map.tilesets.length; j < jl; ++j) {
		var tsInfo = map.tilesets[j];
		tsInfo.lastgid = tsInfo.firstgid + (tsInfo.imagewidth / tsInfo.tilewidth) * (tsInfo.imageheight / tsInfo.tileheight);
		this.tilesets.push(tsInfo);
		var loader = new PIXI.ImageLoader(tsInfo.image, false);
		loader.addEventListener("loaded", tilesetLoaded.bind(this));
		loader.load();
	}

	this.version = map.version;

};

LINK.TiledMap.prototype.tilesetsLoaded = function (map) {
	for (var i = 0, il = map.layers.length; i < il; ++i) {
		map.layers[i].tilewidth = map.tilewidth;
		map.layers[i].tileheight = map.tileheight;
		if (map.layers[i].type === "tilelayer") {
			this.createTileLayer(map.layers[i]);
		}
		if (map.layers[i].type === "objectgroup") {
			this.addLayer(map.layers[i].name);
		}
	}

	this.onLoaded();
};

LINK.TiledMap.prototype.createTileLayer = function (layerJson) {
	var layer = this.addLayer(layerJson.name);
	layer.alpha = layerJson.opacity;
	layer.width = layerJson.width;
	layer.height = layerJson.height;
	layer.visible = layerJson.visible;
	layer.position.x = (layerJson.x * this.tilesize.width) | 0;
	layer.position.y = (layerJson.y * this.tilesize.height) | 0;
	layer.type = layerJson.type;
	var renderTexture = new PIXI.RenderTexture(
		layer.width * this.tilesize.height,
		layer.height * this.tilesize.width);
	for (var index in layerJson.data) {
		var tileIndex = layerJson.data[index];
		if (tileIndex === 0) {
			continue;
		}
		var tileset = this.getTileSet(tileIndex);
		var tileTexture = this.getTileTexture(tileIndex, tileset);
		var tileDisplayObject = new PIXI.Sprite(tileTexture);
		tileDisplayObject.position.x = ((index % layer.width) | 0) * this.tilesize.width;
		tileDisplayObject.position.y = ((index / layer.width) | 0) * this.tilesize.height;
		var doc = new PIXI.DisplayObjectContainer();
		doc.addChild(tileDisplayObject);
		renderTexture.render(doc);
	}
	layer.addChild(new PIXI.Sprite(renderTexture));
};

LINK.TiledMap.prototype.getTileSet = function (index) {
	for (var idx in this.tilesets) {
		var tileset = this.tilesets[idx];
		if (index >= tileset.firstgid && index < tileset.lastgid) return tileset;
	}
	throw new Error("Tileset for index " + index + " not found!");
};

LINK.TiledMap.prototype.getTileTexture = function (index, tileset) {
	if (this.tilecache[index]) {
		return this.tilecache[index];
	}
	var i = index - tileset.firstgid;
	var frame = new PIXI.Rectangle(
	(((i * tileset.tilewidth) % tileset.imagewidth) | 0), (((i * tileset.tilewidth) / tileset.imagewidth) | 0) * tileset.tileheight,
		tileset.tilewidth,
		tileset.tileheight);
	this.tilecache[index] = new PIXI.Texture(tileset.baseTexture, frame);
	return this.tilecache[index];
};

LINK.TiledMap.prototype.onLoaded = function()
{
    this.dispatchEvent({type: "loaded", content: this});
};